/* hiworld.c */
/* Bill Kendrick <bill@newbreedsoftware.com> */
/* August 2013 - June 2015 */


#include "extended_headers/gtia.h"
#define GTIA_READ  (*(struct __x_gtia_read*)0xD000)
#define GTIA_WRITE (*(struct __x_gtia_write*)0xD000)

#include "extended_headers/pbi.h"

#include "extended_headers/pia.h"
#define PIA (*(struct __x_pia*)0xD300)

#include "extended_headers/antic.h"
#define ANTIC (*(struct __x_antic*)0xD400)

#include "extended_headers/pokey.h"
#define POKEY_READ  (*(struct __x_pokey_read*)0xD200)
#define POKEY_WRITE (*(struct __x_pokey_write*)0xD200)

#include "extended_headers/page0.h"
#include "extended_headers/page2.h"

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include "lib/sound.h"

#pragma data-name (push,"FONT")
#include "font.h"
#pragma data-name (pop)

#pragma bss-name ("FONT2")
unsigned char font2[1024];

extern void _ZP_START__[];
extern void _ZP_SIZE__[];
extern void _RAM_START__[];
extern void _RAM_SIZE__[];
extern void _CODE_SIZE__[];
extern void _BSS_SIZE__[];

unsigned char brite = 0;

#pragma optimize (push, off)
void dli(void) {
  DLI_START

  ANTIC.wsync = 1;
  GTIA_WRITE.colbk = (ANTIC.vcount + brite) & 15;

  DLI_END
}
#pragma optimize (pop)

/* From http://atariage.com/forums/topic/233684-cc65-dli-graphics-0/ */

void dli_init(void) {
  unsigned char i;
  unsigned char * DLIST;

  ANTIC.nmien = NMIEN_VBI;

  while (ANTIC.vcount < 124) ;

  DLIST = (unsigned char *) SDLST;
  VDSLST = (unsigned) dli;

  for (i = 6; i < 26; i++) {
    DLIST[i] = DL_CHR40x8x1 | DL_DLI;
  }

  ANTIC.nmien = NMIEN_VBI | NMIEN_DLI;
}

void dli_clear(void) {
  ANTIC.nmien = NMIEN_VBI;
}

unsigned int OldVBI;

#pragma optimize (push, off)
void vbi(void)
{
  COLOR4 = POKEY_READ.random;
  ++brite;

  asm("jmp (%v)", OldVBI);
}

void vbi_init(void)
{
  OldVBI = VVBLKD;
  CRITIC = 1;
  VVBLKD = (unsigned int) vbi;
  CRITIC = 0;
}

void vbi_clear(void)
{
  CRITIC = 1;
  VVBLKD = (unsigned int) OldVBI;
  CRITIC = 0;
}
#pragma optimize (pop)

int main(void) {
  unsigned int i, j;
  int fd;
  unsigned char buf;

  SOUND_INIT();

  printf("\nSome memory declarations:\n");
  printf("ZP   Start %u, Size %u\n", _ZP_START__, _ZP_SIZE__);
  printf("RAM  Start %u, Size %u\n", _RAM_START__, _RAM_SIZE__);
  printf("CODE Size %u\n", _CODE_SIZE__);
  printf("BSS  Size %u\n", _BSS_SIZE__);
  sleep(2);

  printf("\nInline assembly\n");
  asm("ldx %b", 0);
  asm("stx %w", 710);
  sleep(2);

  printf("\nFile access\n");
  fd = open("D1:HIWORLD.AR0", O_RDONLY);
  printf("fd = %d\n", fd);
  if (fd > 0) {
    for (i = 0; i < 50; i++) {
      read(fd, &buf, 1);
      printf("%d ", buf);
    }
    printf("\n");
    close(fd);
  }
  sleep(2);

  printf("\nSwitching font via preloaded segment\n");
  CHBAS = ((unsigned int) &font)/256;
  sleep(2);

  printf("\nSwitching font via empty (BSS) segment\n");
  for (i = 0; i < 128; i++) {
    for (j = 0; j < 8; j++) {
      font2[i * 8 + j] = font[i * 8 + 7 - j];
    }
  }
  CHBAS = ((unsigned int) &font2)/256;
  sleep(2);

  printf("\nLoading font from disk\n");
  fd = open("D1:ECKIG.FNT", O_RDONLY);
  if (fd > 0) {
    read(fd, font2, 1024);
    close(fd);
  } else {
    printf("(Can't do it!)\n");
    CHBAS = CHBASE_DEFAULT;
  }
  sleep(2);

  printf("\nEnabling DLI\n");
  dli_init();
  sleep(2);

  printf("\nEnabling VBI\n");
  vbi_init();
  sleep(2);

  printf("\nSOUND() and lights\n");
  SOUND(2, 100, 12, 4);
  for (i = 0; i < 31; i++) {
    COLOR1 = i << 1;
    SOUND(0, (i + 1) << 3, 10, 15);
    SOUND(1, i << 3, 10, 4);
    printf(".");
  }
  printf("\n");
  SOUND_INIT();
  sleep(2);

  printf("\nDSOUND()\n");
  for (i = 0; i < 65000L; i = i + 512) {
    printf("%7u ", i);
    DSOUND(0, i, 10, 10);
  }
  printf("\n");
  SOUND_INIT();
  sleep(2);

  printf("\nGoodbye!\n");
  vbi_clear();
  dli_clear();
  sleep(2);

  return(0);
}
